@@ -1,7 +1,9 @@
-from .util import old_new_new, NON_DEFAULT_MANAGEMENT_KEY
 import re
+
 import pytest
 
+from .util import NON_DEFAULT_MANAGEMENT_KEY, old_new_new
+
 
 class TestManagementKey:
     def test_change_management_key_force_fails_without_generate(self, ykman_cli, keys):
