@@ -42,6 +42,7 @@ from yubikit.core.otp import (
     modhex_encode,
 )
 from yubikit.core.smartcard import SmartCardConnection
+from yubikit.management import CAPABILITY
 from yubikit.yubiotp import (
     NDEF_TYPE,
     SLOT,
@@ -106,6 +107,13 @@ click_slot_argument = click.argument(
     "slot", type=click.Choice(["1", "2"]), callback=lambda c, p, v: SLOT(int(v))
 )
 
+click_enter_option = click.option(
+    "--enter/--no-enter",
+    default=True,
+    show_default=True,
+    help="send an Enter keystroke after slot output",
+)
+
 
 _WRITE_FAIL_MSG = (
     "Failed to write to the YubiKey. Make sure the device does not "
@@ -191,13 +199,20 @@ def otp(ctx, access_code):
         try:
             access_code = parse_access_code_hex(access_code)
         except Exception as e:
-            ctx.fail(f"Failed to parse access code: {e}")
+            raise CliFail(f"Failed to parse access code: {e}")
 
     ctx.obj["access_code"] = access_code
 
 
-def _get_session(ctx, types=[OtpConnection, SmartCardConnection]):
+def _get_session(ctx, types=None):
     dev = ctx.obj["device"]
+    info = ctx.obj["info"]
+    if not types:
+        if info.version >= (5, 4, 3):
+            # Prefer SmartCardConnection on newer keys
+            types = [SmartCardConnection, OtpConnection]
+        else:
+            types = [OtpConnection, SmartCardConnection]
 
     resolve_scp = ctx.obj.get("scp")
     if resolve_scp:
@@ -208,13 +223,17 @@ def _get_session(ctx, types=[OtpConnection, SmartCardConnection]):
 
     for conn_type in types:
         if dev.supports_connection(conn_type):
-            conn = dev.open_connection(conn_type)
-            ctx.call_on_close(conn.close)
-            if resolve_scp:
-                scp_params = resolve_scp(conn)
-            else:
-                scp_params = None
-            return YubiOtpSession(conn, scp_params)
+            try:
+                conn = dev.open_connection(conn_type)
+                ctx.call_on_close(conn.close)
+                if resolve_scp:
+                    scp_params = resolve_scp(conn)
+                else:
+                    scp_params = None
+                return YubiOtpSession(conn, scp_params)
+            except Exception as e:
+                logger.warning("Failed to open connection: %s", e)
+                continue
 
     raise CliFail(
         "The connection type required for this command is not supported/enabled on the "
@@ -230,8 +249,8 @@ def info(ctx):
     """
     session = _get_session(ctx)
     state = session.get_config_state()
-    slot1 = state.is_configured(1)
-    slot2 = state.is_configured(2)
+    slot1 = state.is_configured(SLOT.ONE)
+    slot2 = state.is_configured(SLOT.TWO)
 
     click.echo(f"Slot 1: {slot1 and 'programmed' or 'empty'}")
     click.echo(f"Slot 2: {slot2 and 'programmed' or 'empty'}")
@@ -248,11 +267,12 @@ def swap(ctx, force):
     Swaps the two slot configurations.
     """
     session = _get_session(ctx)
-    force or click.confirm(
-        "Swap the two slots of the YubiKey?",
-        abort=True,
-        err=True,
-    )
+    if not force:
+        click.confirm(
+            "Swap the two slots of the YubiKey?",
+            abort=True,
+            err=True,
+        )
 
     try:
         session.swap_slots()
@@ -293,6 +313,11 @@ def ndef(ctx, slot, prefix, ndef_type):
 
     try:
         session.set_ndef_configuration(slot, prefix, ctx.obj["access_code"], ndef_type)
+
+        if CAPABILITY.OTP not in info.config.enabled_capabilities[TRANSPORT.NFC]:
+            logger.warning(
+                "NDEF is currently disabled. Enable using 'ykman config nfc -e otp'"
+            )
         click.echo("NDEF configuration updated.")
     except CommandError:
         raise CliFail(_WRITE_FAIL_MSG)
@@ -310,11 +335,12 @@ def delete(ctx, slot, force):
     state = session.get_config_state()
     if not force and not state.is_configured(slot):
         raise CliFail("Not possible to delete an empty slot.")
-    force or click.confirm(
-        f"Do you really want to delete the configuration of slot {slot}?",
-        abort=True,
-        err=True,
-    )
+    if not force:
+        click.confirm(
+            f"Do you really want to delete the configuration of slot {slot}?",
+            abort=True,
+            err=True,
+        )
     try:
         session.delete_slot(slot, ctx.obj["access_code"])
         click.echo(f"Configuration slot {slot} deleted.")
@@ -347,11 +373,7 @@ def delete(ctx, slot, force):
     callback=parse_hex(16),
     help="16 byte secret key",
 )
-@click.option(
-    "--no-enter",
-    is_flag=True,
-    help="don't send an Enter keystroke after emitting the OTP",
-)
+@click_enter_option
 @click.option(
     "-S",
     "--serial-public-id",
@@ -395,7 +417,7 @@ def yubiotp(
     public_id,
     private_id,
     key,
-    no_enter,
+    enter,
     force,
     serial_public_id,
     generate_private_id,
@@ -419,13 +441,15 @@ def yubiotp(
         )
 
     if public_id and serial_public_id:
-        ctx.fail("Invalid options: --public-id conflicts with --serial-public-id.")
+        raise CliFail("Invalid options: --public-id conflicts with --serial-public-id.")
 
     if private_id and generate_private_id:
-        ctx.fail("Invalid options: --private-id conflicts with --generate-public-id.")
+        raise CliFail(
+            "Invalid options: --private-id conflicts with --generate-public-id."
+        )
 
     if key and generate_key:
-        ctx.fail("Invalid options: --key conflicts with --generate-key.")
+        raise CliFail("Invalid options: --key conflicts with --generate-key.")
 
     if not public_id:
         if serial_public_id:
@@ -437,7 +461,7 @@ def yubiotp(
             public_id = modhex_encode(b"\xff\x00" + struct.pack(b">I", serial))
             click.echo(f"Using YubiKey serial as public ID: {public_id}")
         elif force:
-            ctx.fail(
+            raise CliFail(
                 "Public ID not given. Remove the --force flag, or "
                 "add the --serial-public-id flag or --public-id option."
             )
@@ -445,18 +469,18 @@ def yubiotp(
             public_id = click_prompt("Enter public ID")
 
     if len(public_id) % 2:
-        ctx.fail("Invalid public ID, length must be a multiple of 2.")
+        raise CliFail("Invalid public ID, length must be a multiple of 2.")
     try:
-        public_id = modhex_decode(public_id)
+        public_id_bytes = modhex_decode(public_id)
     except ValueError:
-        ctx.fail(f"Invalid public ID, must be modhex ({MODHEX_ALPHABET}).")
+        raise CliFail(f"Invalid public ID, must be modhex ({MODHEX_ALPHABET}).")
 
     if not private_id:
         if generate_private_id:
             private_id = os.urandom(6)
             click.echo(f"Using a randomly generated private ID: {private_id.hex()}")
         elif force:
-            ctx.fail(
+            raise CliFail(
                 "Private ID not given. Remove the --force flag, or "
                 "add the --generate-private-id flag or --private-id option."
             )
@@ -469,7 +493,7 @@ def yubiotp(
             key = os.urandom(16)
             click.echo(f"Using a randomly generated secret key: {key.hex()}")
         elif force:
-            ctx.fail(
+            raise CliFail(
                 "Secret key not given. Remove the --force flag, or "
                 "add the --generate-key flag or --key option."
             )
@@ -477,17 +501,16 @@ def yubiotp(
             key = click_prompt("Enter secret key")
             key = bytes.fromhex(key)
 
-    force or click.confirm(
-        f"Program a YubiOTP credential in slot {slot}?", abort=True, err=True
-    )
+    if not force:
+        click.confirm(
+            f"Program a YubiOTP credential in slot {slot}?", abort=True, err=True
+        )
 
     access_code = ctx.obj["access_code"]
     try:
         session.put_configuration(
             slot,
-            YubiOtpSlotConfiguration(public_id, private_id, key).append_cr(
-                not no_enter
-            ),
+            YubiOtpSlotConfiguration(public_id_bytes, private_id, key).append_cr(enter),
             access_code,
             access_code,
         )
@@ -496,7 +519,7 @@ def yubiotp(
 
     if config_output:
         serial = serial or session.get_serial()
-        csv = format_csv(serial, public_id, private_id, key, access_code)
+        csv = format_csv(serial, public_id_bytes, private_id, key, access_code)
         config_output.write(csv + "\n")
         log_or_echo(
             f"Configuration parameters written to {_fname(config_output)}",
@@ -526,14 +549,10 @@ def yubiotp(
     show_default=True,
     help="keyboard layout to use for the static password",
 )
-@click.option(
-    "--no-enter",
-    is_flag=True,
-    help="don't send an Enter keystroke after outputting the password",
-)
+@click_enter_option
 @click_force_option
 @click.pass_context
-def static(ctx, slot, password, generate, length, keyboard_layout, no_enter, force):
+def static(ctx, slot, password, generate, length, keyboard_layout, enter, force):
     """
     Configure a static password.
 
@@ -547,9 +566,9 @@ def static(ctx, slot, password, generate, length, keyboard_layout, no_enter, for
     session = _get_session(ctx)
 
     if password and len(password) > 38:
-        ctx.fail("Password too long (maximum length is 38 characters).")
+        raise CliFail("Password too long (maximum length is 38 characters).")
     if generate and not length:
-        ctx.fail("Provide a length for the generated password.")
+        raise CliFail("Provide a length for the generated password.")
 
     if not password and not generate:
         password = click_prompt("Enter a static password")
@@ -563,7 +582,7 @@ def static(ctx, slot, password, generate, length, keyboard_layout, no_enter, for
     try:
         session.put_configuration(
             slot,
-            StaticPasswordSlotConfiguration(scan_codes).append_cr(not no_enter),
+            StaticPasswordSlotConfiguration(scan_codes).append_cr(enter),
             ctx.obj["access_code"],
             ctx.obj["access_code"],
         )
@@ -610,14 +629,14 @@ def chalresp(ctx, slot, key, totp, touch, force, generate):
 
     if key:
         if generate:
-            ctx.fail("Invalid options: --generate conflicts with KEY argument.")
+            raise CliFail("Invalid options: --generate conflicts with KEY argument.")
         elif totp:
             key = parse_b32_key(key)
         else:
             key = parse_oath_key(key)
     else:
         if force and not generate:
-            ctx.fail(
+            raise CliFail(
                 "No secret key given. Remove the --force flag, "
                 "set the KEY argument or set the --generate flag."
             )
@@ -641,11 +660,12 @@ def chalresp(ctx, slot, key, totp, touch, force, generate):
             key = parse_oath_key(key)
 
     cred_type = "TOTP" if totp else "challenge-response"
-    force or click.confirm(
-        f"Program a {cred_type} credential in slot {slot}?",
-        abort=True,
-        err=True,
-    )
+    if not force:
+        click.confirm(
+            f"Program a {cred_type} credential in slot {slot}?",
+            abort=True,
+            err=True,
+        )
     try:
         session.put_configuration(
             slot,
@@ -672,8 +692,8 @@ def chalresp(ctx, slot, key, totp, touch, force, generate):
     "--digits",
     type=click.Choice(["6", "8"]),
     default="6",
-    help="number of digits in generated TOTP code (default: 6), "
-    "ignored unless --totp is set",
+    show_default=True,
+    help="number of digits in generated TOTP code, ignored unless --totp is set",
 )
 @click.pass_context
 def calculate(ctx, slot, challenge, totp, digits):
@@ -705,7 +725,7 @@ def calculate(ctx, slot, challenge, totp, digits):
                 challenge = time_challenge(int(challenge))
             except Exception:
                 logger.exception("Error parsing challenge")
-                ctx.fail("Timestamp challenge for TOTP must be an integer.")
+                raise CliFail("Timestamp challenge for TOTP must be an integer.")
     else:  # Challenge is hex
         challenge = bytes.fromhex(challenge)
 
@@ -747,18 +767,15 @@ def parse_modhex_or_bcd(value):
     "--digits",
     type=click.Choice(["6", "8"]),
     default="6",
-    help="number of digits in generated code (default is 6)",
+    show_default=True,
+    help="number of digits in generated code",
 )
 @click.option("-c", "--counter", type=int, default=0, help="initial counter value")
 @click.option("-i", "--identifier", help="token identifier")
-@click.option(
-    "--no-enter",
-    is_flag=True,
-    help="don't send an Enter keystroke after outputting the code",
-)
+@click_enter_option
 @click_force_option
 @click.pass_context
-def hotp(ctx, slot, key, digits, counter, identifier, no_enter, force):
+def hotp(ctx, slot, key, digits, counter, identifier, enter, force):
     """
     Program an HMAC-SHA1 OATH-HOTP credential.
 
@@ -810,9 +827,10 @@ def hotp(ctx, slot, key, digits, counter, identifier, no_enter, force):
             except Exception as e:
                 click.echo(e)
 
-    force or click.confirm(
-        f"Program a HOTP credential in slot {slot}?", abort=True, err=True
-    )
+    if not force:
+        click.confirm(
+            f"Program a HOTP credential in slot {slot}?", abort=True, err=True
+        )
     try:
         session.put_configuration(
             slot,
@@ -820,7 +838,7 @@ def hotp(ctx, slot, key, digits, counter, identifier, no_enter, force):
             .imf(counter)
             .token_id(token_id, mh1, mh2)
             .digits8(int(digits) == 8)
-            .append_cr(not no_enter),
+            .append_cr(enter),
             ctx.obj["access_code"],
             ctx.obj["access_code"],
         )
@@ -843,12 +861,7 @@ def hotp(ctx, slot, key, digits, counter, identifier, no_enter, force):
 @click.option(
     "--delete-access-code", is_flag=True, help="remove access code from the slot"
 )
-@click.option(
-    "--enter/--no-enter",
-    default=True,
-    show_default=True,
-    help="send an Enter keystroke after slot output",
-)
+@click_enter_option
 @click.option(
     "-p",
     "--pacing",
@@ -860,10 +873,14 @@ def hotp(ctx, slot, key, digits, counter, identifier, no_enter, force):
 @click.option(
     "--use-numeric-keypad",
     is_flag=True,
-    show_default=True,
     help="use scancodes for numeric keypad when sending digits "
     "(helps for some keyboard layouts)",
 )
+@click.option(
+    "--serial-usb-visible",
+    is_flag=True,
+    help="make the serial number visible in the USB descriptor",
+)
 def settings(
     ctx,
     slot,
@@ -872,6 +889,7 @@ def settings(
     enter,
     pacing,
     use_numeric_keypad,
+    serial_usb_visible,
     force,
 ):
     """
@@ -883,7 +901,7 @@ def settings(
     session = _get_session(ctx)
 
     if new_access_code and delete_access_code:
-        ctx.fail("--new-access-code conflicts with --delete-access-code.")
+        raise CliFail("--new-access-code conflicts with --delete-access-code.")
 
     if delete_access_code and not ctx.obj["access_code"]:
         raise CliFail(
@@ -906,16 +924,17 @@ def settings(
         try:
             new_access_code = parse_access_code_hex(new_access_code)
         except Exception as e:
-            ctx.fail("Failed to parse access code: " + str(e))
+            raise CliFail("Failed to parse access code: " + str(e))
         if ctx.obj["info"].pin_complexity and len(set(new_access_code)) < 2:
             raise CliFail("Access code does not meet complexity requirement.")
 
-    force or click.confirm(
-        f"Update the settings for slot {slot}? "
-        "All existing settings will be overwritten.",
-        abort=True,
-        err=True,
-    )
+    if not force:
+        click.confirm(
+            f"Update the settings for slot {slot}? "
+            "All existing settings will be overwritten.",
+            abort=True,
+            err=True,
+        )
 
     pacing_bits = int(pacing or "0") // 20
     pacing_10ms = bool(pacing_bits & 1)
@@ -927,7 +946,8 @@ def settings(
             UpdateConfiguration()
             .append_cr(enter)
             .use_numeric(use_numeric_keypad)
-            .pacing(pacing_10ms, pacing_20ms),
+            .pacing(pacing_10ms, pacing_20ms)
+            .serial_usb_visible(serial_usb_visible),
             new_access_code,
             ctx.obj["access_code"],
         )
